from django import template
from django.core.urlresolvers import reverse
from django.conf import settings
from menuz.models import Menuz, MenuzItem
from menuz.registry import menuz
from menuz.utils import get_menu_by_position

register = template.Library()

@register.inclusion_tag('admin/menuz/menuz/menu_builder.html')
def menu_builder(object_id):
    """
    Menu Builder tags to use in django admin menu manager.
    """
    data = {
        'menu_id': object_id
        }

    #get available menus
    available_menus = []
    for meta in menuz.registry:
        source = {
            'type': meta['model'].__name__.lower(),
            'name' :meta['model']._meta.verbose_name_plural.title()
        }
        if meta.get('custom_source',''):
            source['queryset'] = meta['custom_source']()
        else:
            source['queryset'] = meta['model'].objects.all()
        available_menus.append(source)
    data['available_menus'] = available_menus

    #get Inner links
    if hasattr(settings, 'AVAILABLE_INNERLINKS'):
        data['innerlinks'] = getattr(settings, 'AVAILABLE_INNERLINKS')

    #get MenuItems
    menu_items = MenuzItem.objects.filter(menu__id=object_id).order_by('order')
    data['menu_items'] = menu_items

    return data



"""
Get Menu tag to use in front end and return value as context to give template designer
flexibility to style menu items them self.
returned value:
1. [varname]
2. [varname_title]
"""
@register.tag
def get_menu(parser, token):
    args = token.split_contents()

    if len(args) != 4:
        raise template.TemplateSyntaxError('get_menu tag usage not valid!. Usage : get_menu [menu_id] as [varname]')

    return MenuNode(args[1], args[3])

class MenuNode(template.Node):
    def __init__(self, position_id, varname):
        self.position_id = position_id
        self.varname = varname

    def render(self, context):
        title, items = get_menu_by_position(self.position_id)
        context[self.varname+'_title'] = title
        context[self.varname] = items
        return ''

@register.simple_tag
def list_menu(position_id):
    """
    Simple and generic menu tags to print menu items as a html list.
    this tags must be surrounded by <ul> tags.
    example:
    <ul>
    {% list_menu 'top_menu' %}
    </ul>
    """
    title, items = get_menu_by_position(position_id)
    output = []
    for item in items:
        output.append(u'<li><a href="%s" title="%s">%s</a></li>' % (item['url'], item['title'], item['title']))
    return u'\n'.join(output)

@register.filter
def truncatechars_right(value, length):
    if len(value) <= length:
        return value
    else:
        return u'%s...' % value[:length-3]




from django.contrib import admin
from menuz.models import Menuz

class MenuzAdmin(admin.ModelAdmin):
    list_display = ('title','position')
    save_on_top = True

admin.site.register(Menuz, MenuzAdmin)




from django.http import HttpResponse
from django.utils import simplejson
from django.utils.html import strip_tags
from django.contrib.auth.decorators import login_required

from menuz.models import Menuz, MenuzItem
from menuz.forms import CustomMenuForm
from menuz.registry import get_menuz_object_model

@login_required
def add_menuz(request):
    if request.is_ajax():
        if request.method == 'POST':
            mtype = request.POST.get('mtype','')
            menu_id = request.POST.get('menu_id',0)
            menu = Menuz.objects.get(pk=menu_id)

            #if menu type custom
            if mtype == 'custom':
                customform = CustomMenuForm(request.POST)
                if customform.is_valid():

                    menuitem = MenuzItem()
                    menuitem.menu = menu
                    menuitem.content_type = mtype
                    menuitem.title = strip_tags(customform.cleaned_data['custom_title'])
                    menuitem.url = customform.cleaned_data['custom_url']
                    menuitem.save()
                    data = {
                        'id': menuitem.id,
                        'title': menuitem.title,
                        'url': menuitem.url,
                        'content_type': menuitem.content_type,
                        'content_id': menuitem.content_id
                    }
                    menu_data = [data]
                    return HttpResponse(simplejson.dumps({'status':'success', 'menu_data': menu_data}),
                                        content_type='application/javascript; charset=utf-8;')
                else:
                    fields = [field for field in customform.errors]
                    return HttpResponse(simplejson.dumps({'status':'failed', 'fields':fields}),
                                        content_type='application/javascript; charset=utf-8;')

            #if menu type Innerlink, similar to custom but different treatment
            elif mtype == 'innerlink':

                links = request.POST.getlist(mtype)

                if links and menu:
                    menu_data = []
                    for link in links:
                        link_split = link.split('$')

                        menuitem = MenuzItem()
                        menuitem.menu = menu
                        menuitem.content_type = mtype
                        menuitem.title = strip_tags(link_split[1])
                        menuitem.url = strip_tags(link_split[0])
                        menuitem.save()

                        data = {
                            'id': menuitem.id,
                            'title': menuitem.title,
                            'url': menuitem.url,
                            'content_type': menuitem.content_type,
                            'content_id': menuitem.content_id
                        }
                        menu_data.append(data)
                    return HttpResponse(simplejson.dumps({'status':'success', 'menu_data': menu_data}),
                                        content_type='application/javascript; charset=utf-8;')

            #if menu type Model Menu
            else:
                obj_ids = request.POST.getlist(mtype)
                model = get_menuz_object_model(mtype)

                if obj_ids and menu and model:
                    queryset = model.objects.filter(pk__in=obj_ids)
                    menu_data = []
                    for obj in queryset:

                        menuitem = MenuzItem()
                        menuitem.menu = menu
                        menuitem.content_type = mtype
                        menuitem.content_id = obj.pk
                        menuitem.title = obj.__unicode__()
                        menuitem.save()

                        data = {
                            'id': menuitem.id,
                            'title': menuitem.title,
                            'url': menuitem.url,
                            'content_type': menuitem.content_type,
                            'content_id': menuitem.content_id
                        }
                        menu_data.append(data)
                    return HttpResponse(simplejson.dumps({'status':'success', 'menu_data': menu_data}),
                                        content_type='application/javascript; charset=utf-8;')

            return HttpResponse(simplejson.dumps({'status':'failed'}),
                                    content_type='application/javascript; charset=utf-8;')


@login_required
def reorder_menuz(request):
    if request.is_ajax():
        if request.method == 'POST':
            menus = request.POST.get('order','')
            if menus:
                try:
                    menu_ids = menus.split(',')
                    count = 1
                    #main loop to re-set menu order value
                    for m_id in menu_ids:
                        try:
                            menu = MenuzItem.objects.get(pk=m_id)
                            menu.order = count
                            menu.save()
                            count += 1
                        except:
                            pass
                    return HttpResponse(simplejson.dumps({'status':'success'}),
                                        content_type='application/javascript; charset=utf-8;')
                except:
                    pass
            return HttpResponse(simplejson.dumps({'status':'Failed re-ordering menus!'}),
                                content_type='application/javascript; charset=utf-8;')
@login_required
def delete_menuz(request):
    if request.is_ajax():
        item_id = request.POST.get('item_id',0)
        if item_id:
            try:
                menu_obj = MenuzItem.objects.get(pk=item_id)
                menu_obj.delete()
                return HttpResponse(simplejson.dumps({'status':'success'}),
                            content_type='application/javascript; charset=utf-8;')
            except MenuzItem.DoesNotExist:
                pass

        return HttpResponse(simplejson.dumps({'status':'failed'}),
                            content_type='application/javascript; charset=utf-8;')
@login_required
def update_menuz(request):
    if request.is_ajax():
        if request.method == 'POST':
            mtype = request.POST.get('mtype', '')
            item_id = request.POST.get('item_id', 0)

            status = False
            #try to get menu item object
            item = None
            try:
                item = MenuzItem.objects.get(pk=item_id)
            except MenuzItem.DoesNotExist:
                pass

            #if menu item found
            if item:
                title = request.POST.get(mtype+'_title_'+item_id, '')
                #if menu type custom link
                #update title and Url
                if mtype == 'custom':
                    url = request.POST.get(mtype+'_url_'+item_id, '')
                    if title and url:
                        item.title = strip_tags(title)
                        item.url = strip_tags(url)
                        item.save()
                        status = True
                else:
                    if title:
                        item.title = strip_tags(title)
                        item.save()
                        status = True
            if status:
                return HttpResponse(simplejson.dumps({'status':'success', 'new_title': item.title}),
                                content_type='application/javascript; charset=utf-8;')
            else:
                return HttpResponse(simplejson.dumps({'status':'failed'}),
                                content_type='application/javascript; charset=utf-8;')




from django.core.urlresolvers import reverse
from menuz.models import Menuz, MenuzItem
from menuz.registry import menuz
from menuz.registry import get_menuz_object_model


# used in conjunction with egocms app
# part of Egomedia Bali's inhouse CMS project
egosetting_available = True
try:
    from egosettings.egosetting import get_setting
except ImportError:
    egosetting_available = False
    

def get_menu_by_position(position):
    """
    Return all menu items based on position ID.
    """
    menu_title = ''
    all_items = []
    #get all menu items
    try:
        menu = Menuz.objects.get(position__exact=position)
        menu_title = menu.title
        menu_contents = menu.menuzitem_set.all()
        #get all menu data, and return title and links only 
        
        for obj in menu_contents:
            if obj.content_type == 'custom' or obj.content_type == 'innerlink':
                all_items.append({'url': obj.url, 'title': obj.title})
            else:
                if get_menuz_object_model(obj.content_type):
                    model = get_menuz_object_model(obj.content_type)
                    try:
                        item = model.objects.get(pk=obj.content_id)
                        
                        # works with egocms
                        # this will check if current page setted as homepage_id
                        # if yes, then display site base url
                        if egosetting_available:
                            homepage_id = get_setting('site_homepage')
                            if homepage_id == item.id:
                                all_items.append({'url': reverse('index'), 'title': obj.title})
                            else:
                                all_items.append({'url': item.get_absolute_url(), 'title': obj.title})
                        else:
                            all_items.append({'url': item.get_absolute_url(), 'title': obj.title})
                            
                    except Exception, err:
                        print err
    except Exception, err:
        print err
    #return a tuple of title and its items
    return (menu_title, all_items)




from django.conf.urls.defaults import patterns, include, url

urlpatterns = patterns('menuz.views',
    url(r'^menuz/add_menuz/','add_menuz' , name='add_menuz'),
    url(r'^menuz/reorder/','reorder_menuz' , name='reorder_menuz'),
    url(r'^menuz/delete/','delete_menuz' , name='delete_menuz'),
    url(r'^menuz/update/','update_menuz' , name='update_menuz'),
)




"""
This section take its concept from django admin site.
with similar aproach and implementation.

Implementation:
in project root urls.py add folowing code:


from menuz import registry
registry.autodiscover()

This will try to import all file in apps that named menu.py.
"""

class MenuzRegistry(object):

    def __init__(self):
        self.registry = []

    def register(self, model, custom_source=None):
        if model:
            if model in self.registry:
                pass
            else:
                meta = {
                    'type' : model.__name__.lower(),
                    'model': model,
                    'custom_source': custom_source
                }
                self.registry.append(meta)

menuz = MenuzRegistry()

def autodiscover():

    import copy
    from django.conf import settings
    from django.utils.importlib import import_module

    for app in settings.INSTALLED_APPS:
        try:
            before_import_registry = copy.copy(menuz.registry)
            import_module('%s.menu' % app)
        except:
            menuz.registry = before_import_registry

def get_menuz_object_model(content_type):
    for menu in menuz.registry:
        if menu['type'] == content_type:
            return menu['model']
            break
    return False



from django.db import models
from django.conf import settings
from django.utils.translation import ugettext_lazy as _

class Menuz(models.Model):
    title = models.CharField(_('Menu Title'), max_length=50)
    position = models.CharField(_('Menu Position'), max_length=100, choices=settings.AVAILABLE_MENUS, help_text=_('This is position of the menu.'))

    def __unicode__(self):
        return self.title

    class Meta:
        verbose_name = _('Menu')
        verbose_name_plural = _('Menus')

class MenuzItem(models.Model):
    menu = models.ForeignKey(Menuz)
    content_type = models.CharField(max_length=50)
    content_id = models.IntegerField(blank=True, null=True)
    title = models.CharField(max_length=250)
    url = models.URLField(blank=True, null=True)
    parent = models.ForeignKey('self', blank=True, null=True)
    order = models.IntegerField(blank=True, null=True, default=0)

    def __unicode__(self):
        return self.title

    class Meta:
        ordering = ('order',)




#!/usr/bin/env python
# -*- coding: utf-8 -*-

from setuptools import setup, find_packages

setup(
    name = "django-menuz",
    version = "0.2.0",
    url = 'https://github.com/ekaputra07/django-menuz',
	download_url = 'https://github.com/ekaputra07/django-menuz/downloads',
    license = 'BSD',
    description = "Drag and drop menu manager for Django.",
    author = 'Eka Putra',
    author_email = 'ekaputra@balitechy.com',
    packages = find_packages(),
    include_package_data = True,
    zip_safe = False,
    classifiers = [
        'Development Status :: 3 - Alpha',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Internet :: WWW/HTTP',
    ]
)

